{% extends 'base.html' %}

{% block content %}
<div class="container-fluid mt-0">
    <h1>{{ _("Delete User") }}</h1>
    <p>{{ _("Are you sure you want to delete the user") }} '{{ user.name }} {{ user.surname }}'?</p>
    <form method="POST" action="{{ url_for('delete_user', user_id=user.user_id) }}">
        <input type="submit" value="{{ _("Yes, delete user") }}" class="btn btn-danger" />
    </form>
    <a href="{{ url_for('view_users') }}" class="btn btn-secondary">{{ _("No, go back") }}</a>
</div>
{% endblock %}
