"""Maintenance: added short lcd message

Revision ID: 3cb12dfd8d6e
Revises: e544554c7d29
Create Date: 2024-04-24 22:17:20.179783

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '3cb12dfd8d6e'
down_revision: Union[str, None] = 'e544554c7d29'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('maintenances', sa.Column('lcd_message', sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('maintenances', 'lcd_message')
    # ### end Alembic commands ###
