"""Boards: added columns for serial number and free heap

Revision ID: e544554c7d29
Revises: 123b2cca325f
Create Date: 2024-04-21 13:24:54.520194

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = 'e544554c7d29'
down_revision: Union[str, None] = '123b2cca325f'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('boards', sa.Column('serial', sa.String(), nullable=True))
    op.add_column('boards', sa.Column('heap', sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('boards', 'heap')
    op.drop_column('boards', 'serial')
    # ### end Alembic commands ###
