{% extends 'base.html' %}

{% block content %}
    <h1>{% block title %} {{ _("Home") }} {% endblock %}</h1>
    <h1>{{ _("This is a web application to manage machines in a Fab Lab environment.") }}</h1>
    <h3>{{ utc_dt }}</h3>
{% endblock %}