{% extends 'base.html' %}

{% block content %}
<div class="container-fluid mt-0">
    <h1>Reset password</h1>
    <p>Are you sure you want to send reset password e-mail for the user '{{ user.name }} {{ user.surname }} ({{ user.email }})'?</p>
    <form method="POST" action="{{ url_for('reset_user', user_id=user.user_id) }}">
        <input type="submit" value="Yes, send e-mail" class="btn btn-danger" />
    </form>
    <a href="{{ url_for('view_users') }}" class="btn btn-secondary">{{ _("No, go back") }}</a>
</div>
{% endblock %}
