"""Added replay columns for Use and Intervention

Revision ID: 3f3f601505a6
Revises: dfd2b0db0016
Create Date: 2024-05-30 23:09:40.723754

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "3f3f601505a6"
down_revision: Union[str, None] = "dfd2b0db0016"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("interventions", sa.Column("replay", sa.Boolean(), nullable=True))
    op.add_column("uses", sa.Column("replay", sa.Boolean(), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("uses", "replay")
    op.drop_column("interventions", "replay")
    # ### end Alembic commands ###
