{% extends 'base.html' %}

{% block content %}
<div class="container-fluid mt-0">
    <h1>{{ _("Delete Maintenance") }}</h1>
    <p>{{ _("Are you sure you want to delete this maintenance?") }}</p>
    <form method="POST" action="{{ url_for('delete_maintenance', maintenance_id=maintenance.maintenance_id) }}">
        <input type="submit" value="{{ _("Yes, delete maintenance") }}" class="btn btn-danger" />
    </form>
    <a href="{{ url_for('maintenances') }}" class="btn btn-secondary mt-3">{{ _("No, go back") }}</a>
</div>
{% endblock %}
