{% extends 'base.html' %}

{% block content %}
<div class="container-fluid mt-0">
    <h1>{{ _("Add Role") }}</h1>
    <form method="POST" action="{{ url_for('add_role') }}">
        <div class="mb-3">
            <label for="role_name" class="form-label">{{ _("Role Name") }}:</label>
            <input type="text" id="role_name" name="role_name" class="form-control" required />
        </div>
        <div class="mb-3 form-check">
            <input type="checkbox" id="backend_admin" name="backend_admin" class="form-check-input" />
            <label for="backend_admin" class="form-check-label">{{ _("Can admin backend") }}</label>
        </div>
        <div class="mb-3 form-check">
            <input type="checkbox" id="maintenance" name="maintenance" class="form-check-input" />
            <label for="maintenance" class="form-check-label">{{ _("Can perform maintenance") }}</label>
        </div>
        <div class="mb-3 form-check">
            <input type="checkbox" id="authorize_all" name="authorize_all" class="form-check-input" />
            <label for="authorize_all" class="form-check-label">{{ _("Authorize all") }}</label>
        </div>
        <input type="submit" value="{{ _("Add Role") }}" class="btn btn-primary" />
    </form>
</div>
{% endblock %}