{% extends 'base.html' %}

{% block content %}
<div class="container">
    <form method="POST" class="mt-5">
        <div class="form-group">
            <label for="email">{{ _("Email") }}:</label>
            <input type="text" class="form-control" id="email" name="email" required>
        </div>
        <div class="form-group">
            <label for="password">{{ _("Password") }}:</label>
            <input type="password" class="form-control" id="password" name="password" required>
        </div>
        <button type="submit" class="btn btn-primary">{{ _("Login") }}</button>
    </form>
    <a href="{{ url_for('forgot_password') }}">{{ _("Forgot your password?") }}</a>
</div>
{% endblock %}
