from . import (
    webapplication,
    authentication,
    routes_authorizations,
    routes_interventions,
    routes_languages,
    routes_machines,
    routes_machinetypes,
    routes_maintenance,
    routes_roles,
    routes_users,
    routes_uses,
    routes_system,
)
